var nodeSize = 8;
var selectSize = 12;
var startendSize = 14;

function initGraph(graph, vertices, edges, width, height) {
    // calculate the transformation
    var minx = 999999999;
    var maxx = -999999999;
    var miny = 999999999;
    var maxy = -999999999;

    graph.nodes.forEach(function (d) {
        minx = Math.min(minx, d.x);
        maxx = Math.max(maxx, d.x);
        miny = Math.min(miny, d.y);
        maxy = Math.max(maxy, d.y);
    });

    var border = 0.07;

    graph.scale = (1-2*border) * width / Math.max(maxx - minx, maxy - miny);
    graph.xtrans = - minx * graph.scale + border * width + ((1-2*border) * width - maxx * graph.scale + minx * graph.scale) / 2;
    graph.ytrans = - miny * graph.scale + border * height + ((1-2*border) * height - maxy * graph.scale + miny * graph.scale) / 2;

    console.log(width,height);
    // create vertices
    graph.nodes.forEach(function (d) {
        var v = new Vertex(d.id, graph.xtrans + graph.scale * d.x, graph.ytrans + graph.scale * d.y,false);
        vertices.push(v);
    });

    // create edges
    graph.links.forEach(function (d) {
        var e = new Edge(d.id, vertices[d.source], vertices[d.target]);
        edges.push(e);
    });
}

function basicLayout(svg, vertices, edges) {

    // highlighted edges
    svg.append("g").attr("id","linksHighlighted");

    // highlighted vertices
    svg.append("g").attr("id","nodesHighlighted");

    // selected edges
    svg.append("g").attr("id","linksSelected");

    // selected vertices
    svg.append("g").attr("id","nodesSelected");

    // mark start and endpoint
    svg.append("g").attr("id","nodesStartEnd");



    // draw edges
    var container = svg.append("g")
        .attr("id", "links");

    var edgedata = [];
    edges.forEach(function (d) {
        edgedata.push(d);
        edgedata.push(d);
    });

    link = container.selectAll(".links").data(edgedata).enter();

    link.append("line")
        .attr("class", function (d, i) {
            if (i % 2 === 0) {
                return "link linkHalo";
            } else {
                return "link linkNormal";
            }
        })
        .attr("id", function(d, i) {
            if (i % 2 === 0) {
                return "halo" + d.id;
            } else {
                return d.id;
            } })
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });

    // select vertices
    container = svg.append("g")
        .attr("id", "nodes");

    node = container.selectAll(".node")
        .data(vertices)
        .enter().append("g")
        .attr("class", "node")
        .attr("id", function(d) { return "n" + d.id; })
        .attr("transform", function(d) {
            return "translate(" + d.x + "," + d.y + ")"; });

    // draw vertices
    node.append("circle")
        .attr("r", nodeSize)
}



function Vertex(id, x, y, selected) {
    this.x = x;
    this.y = y;
    this.origx = x;
    this.origy = y;
    this.id = id;
    this.index = id;
    this.selected = selected;
    this.highlighted = false;
    this.path_degree = 0;
    this.highlight_degree = 0;
    // -1: none; 0: s; 1: t
    this.component = -1;
    this.marked = false;
}


function Edge(id,s,t) {
    this.id = "e" + id;
    this.t =t;
    this.s =s;
    this.target = t;
    this.source =s;
    this.highlighted = false;
    this.selected = false;
    this.string = "(" + s.id + "," + t.id + ")"
}

function getAdjMatrix(v,e)
{
    var A = [];
    for (var i=0;i<v.length;i++)
    {
        var row = [];
        for (var j = 0; j < v.length; j++) row.push(0);
        A.push(row);
    }

    for (i=0;i< e.length;i++){
        var s = e[i].s.id;
        var t = e[i].t.id;
        A[s][t]=e[i];
        A[t][s]=e[i];
        A[s][s]--;
        A[t][t]--;
    }
    return A;
}

function getAdjList(v,e)
{
    var A = [];
    for (var i=0;i<v.length;i++)
    {
        var row = [];
        A.push(row);
    }

    for (i=0;i< e.length;i++){
        var s = e[i].s.id;
        var t = e[i].t.id;
        A[s].push(e[i]);
        A[t].push(e[i]);
    }
    return A;
}

function updateDrawing(svg) {
    svg.select("#nodesHighlighted")
        .selectAll("circle")
        .transition()
        .attr("transform", function(d) {
            return "translate(" + d.x + "," + d.y + ")"; });
    svg.select("#linksHighlighted")
        .selectAll("line")
        .transition()
        .attr("x1", function(d){return d.s.x;})
        .attr("x2", function(d){return d.t.x;})
        .attr("y1", function(d){return d.s.y;})
        .attr("y2", function(d){return d.t.y;});
    svg.select("#linksHighlighted")
        .selectAll("path")
        .transition()
        .attr("d", function(d){
            var points = [[d.s.x,d.s.y], [(d.s.x+d.t.x)/2,(d.s.y+d.t.y)/2], [d.t.x,d.t.y]];
            return d3.line().curve(d3.curveBasis)(points);
        })
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
    svg.select("#links")
        .selectAll("line")
        .transition()
        .attr("x1", function(d){return d.s.x;})
        .attr("x2", function(d){return d.t.x;})
        .attr("y1", function(d){return d.s.y;})
        .attr("y2", function(d){return d.t.y;});
    svg.select("#links")
        .selectAll("path")
        .transition()
        .attr("d", function(d){
            var points = [[d.s.x,d.s.y], [(d.s.x+d.t.x)/2,(d.s.y+d.t.y)/2], [d.t.x,d.t.y]];
            return d3.line().curve(d3.curveBasis)(points);
        })
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
    svg.select("#nodes")
        .selectAll("g")
        .transition()
        .attr("transform", function(d) {
            return "translate(" + d.x + "," + d.y + ")"; });
}

function addNode(svg, node) {
        // select vertices
    svg.select("#nodes")
        .datum(node)
        .append("g")
        .attr("class", "node")
        .attr("id", function(d) { return "n" + d.id; })
        .attr("transform", function(d) {
            return "translate(" + d.x + "," + d.y + ")"; })
        .append("circle")
        .attr("r",0)
        .transition()
        .attr("r", nodeSize);
}

function removeNode(svg, node) {
    svg.select("#nodes")
        .select("#n" + node.id)
        .transition()
        .attr("r",0)
        .remove();
}

function addEdge(svg, edge) {
    // draw edges
    var container = svg.select("#links");

    var edgedata = [edge, edge];

    container.selectAll(".links")
        .data(edgedata)
        .enter()
        .append("line")
        .attr("class", function (d, i) {
            if (i % 2 === 0) {
                return "link linkHalo";
            } else {
                return "link linkNormal";
            }
        })
        .attr("id", function(d, i) {
            if (i % 2 === 0) {
                return "halo" + d.id;
            } else {
                return d.id;
            } })
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.s.x; })
        .attr("y2", function(d) { return d.s.y; })
        .transition()
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
}

function removeEdge(svg, edge) {
    var container = svg.select("#links");
    container.select("#halo"+edge.id)
        .transition()
        .attr("x2", edge.s.x)
        .attr("y2", edge.s.y)
        .remove();
    container.select("#"+edge.id)
        .transition()
        .attr("x2", edge.s.x)
        .attr("y2", edge.s.y)
        .remove();
}

function addCurvedEdge(svg, edge, direction) {
    // draw edges
    var container = svg.select("#links");

    var lineGenerator = d3.line()
        .curve(d3.curveBasis);

    var points = [];
    if (direction)
        points = [[edge.s.x,edge.s.y], [edge.s.x,edge.t.y], [edge.t.x,edge.t.y]];
    else
        points = [[edge.s.x,edge.s.y], [edge.t.x,edge.s.y], [edge.t.x,edge.t.y]];

    var lh = container.data([edge])
        .append("path")
        .attr("class", "link linkHalo")
        .attr("id", function(d, i) {
            return "halo" + d.id;
            })
        .attr("d", lineGenerator(points))
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
    var l = container.data([edge])
        .append("path")
        .attr("class", "link linkNormal")
        .attr("id", function(d, i) {
            return d.id;
        })
        .attr("d", lineGenerator(points))
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });

    var totalLength = l.node().getTotalLength();
    lh.attr("stroke-dasharray", totalLength + " " + totalLength)
        .attr("stroke-dashoffset", totalLength)
        .transition()
        .attr("stroke-dashoffset", 0);
    l.attr("stroke-dasharray", totalLength + " " + totalLength)
        .attr("stroke-dashoffset", totalLength)
        .transition()
        .attr("stroke-dashoffset", 0);
}

function removeCurvedEdge(svg, edge) {
    var container = svg.select("#links");
    var totalLength = container.select("#"+edge.id).node().getTotalLength();
    container.select("#halo"+edge.id)
        .transition()
        .attr("stroke-dashoffset", totalLength)
        .remove();
    container.select("#"+edge.id)
        .transition()
        .attr("stroke-dashoffset", totalLength)
        .remove();
}

function curve(svg, edge, direction) {
    // draw edges
    var container = svg.select("#links");
    var lineGenerator = d3.line()
        .curve(d3.curveBasis);
    var points = [];
    if (direction)
        points = [[edge.s.x,edge.s.y], [edge.s.x,edge.t.y], [edge.t.x,edge.t.y]];
    else
        points = [[edge.s.x,edge.s.y], [edge.t.x,edge.s.y], [edge.t.x,edge.t.y]];

    container.select("#halo"+edge.id)
        .transition()
        .attr("d", lineGenerator(points))
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
    container.select("#"+edge.id)
        .transition()
        .attr("d", lineGenerator(points))
        .attr("x1", function(d) { return d.s.x; })
        .attr("y1", function(d) { return d.s.y; })
        .attr("x2", function(d) { return d.t.x; })
        .attr("y2", function(d) { return d.t.y; });
}

