function highlightedLayout(svg, adjList, selected_path, path_start, path_target) {


    path_start.component = 0;
    path_target.component = 1;
    path_start.selected = true;
    path_target.selected = true;
    path_start.path_degree++;
    path_target.path_degree++;
    selected_path[0].push(path_start);
    selected_path[1].push(path_target);

    // mark start and endpoint
    var container = svg.select("#nodesStartEnd");
    container.append("circle")
        .attr("class", "nodeStartEnd")
        .attr("id", "nse" + path_start.id)
        .attr("transform", "translate(" + path_start.x + "," + path_start.y + ")")
        .attr("r", startendSize);
    container.append("circle")
        .attr("class", "nodeSelect")
        .attr("id", "ns" + path_start.id)
        .attr("transform", "translate(" + path_start.x + "," + path_start.y + ")")
        .attr("r", startendSize);
    container.append("circle")
        .attr("class", "nodeStartEnd")
        .attr("class", "nodeStartEnd")
        .attr("id", "nse" + path_target.id)
        .attr("transform", "translate(" + path_target.x + "," + path_target.y + ")")
        .attr("r", startendSize);
    container.append("circle")
        .attr("class", "nodeSelect")
        .attr("id", "ns" + path_target.id)
        .attr("transform", "translate(" + path_target.x + "," + path_target.y + ")")
        .attr("r", startendSize);


    // highlight edges incident to path-source and path-target
    findNewHighlights(svg, path_start, adjList);
    findNewHighlights(svg, path_target, adjList);
}


function highlightNode(svg, v, c) {
    var cm = c || "";
    v.highlighted = true;
    svg.select("#nodesHighlighted")
        .datum(v)
        .append("circle")
        .attr("class", "nodeHighlight" + cm)
        .attr("id", "nh" + v.id)
        .attr("transform", "translate(" + v.x + "," + v.y + ")")
        .attr("r", nodeSize)
        .transition()
        .attr("r", selectSize)
}

function highlightLink(svg, e, c) {
    var cm = c || "";
    e.highlighted = true;
    svg.select("#linksHighlighted")
        .datum(e)
        .append("line")
        .attr("class", "linkHighlight" + cm)
        .attr("id", "lh" + e.id)
        .attr("x1", e.s.x)
        .attr("y1", e.s.y)
        .attr("x2", e.s.x)
        .attr("y2", e.s.y)
        .transition()
        .attr("x2", e.t.x)
        .attr("y2", e.t.y);
    // update highlight degrees
    e.s.highlight_degree++;
    e.t.highlight_degree++;
}

function clearHighlights(svg, edges) {
    svg.select("#nodesHighlighted")
        .selectAll("circle")
        .transition()
        .style("opacity",0)
        .remove();
    svg.select("#linksHighlighted")
        .selectAll("line")
        .transition()
        .style("opacity",0)
        .remove();
    edges.forEach(function(e) {
        e.highlighted = false;
        e.s.highlight_degree = 0;
        e.t.highlight_degree = 0;
    })
}

function selectNode(svg, v, selected_path) {
    v.selected = true;
    // don't update path degree here because it's done at the link
    svg.select("#nodesSelected")
        .append("circle")
        .attr("class", "nodeSelect")
        .attr("id", "ns" + v.id)
        .attr("transform", "translate(" + v.x + "," + v.y + ")")
        .attr("r", nodeSize)
        .transition()
        .attr("r", selectSize);
    selected_path[v.component].push(v)
}

function selectLink(svg, e) {
    e.selected = true;
    svg.select("#linksSelected")
        .append("line")
        .attr("class", "linkSelect")
        .attr("id", "ls" + e.id)
        .attr("x1", e.s.x)
        .attr("y1", e.s.y)
        .attr("x2", e.s.x)
        .attr("y2", e.s.y)
        .transition()
        .attr("x2", e.t.x)
        .attr("y2", e.t.y);
    // update path degrees
    e.s.path_degree++;
    e.t.path_degree++;
}

// highlight edges incident to t
function findNewHighlights(svg, t, adjList) {
    var tedges = adjList[t.id];
    var tedge;
    for (var i = 0; i < tedges.length; i++) {
        tedge = tedges[i];
        if (!tedge.highlighted && !tedge.selected) {
            // u is Vertex on tedge=(t,u) other than t
            u = (tedge.s.id === t.id) ? tedge.t : tedge.s;

            if (!u.selected) {
                // highlight the edge
                highlightLink(svg, tedge, 5);

                // highlight u
                if (!u.highlighted) {
                    highlightNode(svg, u, 5);
                }
            }
        }
    }
}

function unhighlightLink(svg,e) {
    svg.select("#linksHighlighted")
        .select("#lh" + e.id)
        .transition()
        .attr("x2",e.s.x)
        .attr("y2",e.s.y)
        .remove();
    e.s.highlight_degree--;
    e.t.highlight_degree--;
}

function unhighlight(svg, e) {
    if (e.highlighted) {
        e.highlighted = false;
        svg.select("#linksHighlighted")
            .select("#lh" + e.id)
            .attr("x2",e.s.x)
            .attr("y2",e.s.y)
            .remove();
        // remove highlighted from e.s and e.t
        e.s.highlight_degree--;
        if (e.s.highlight_degree === 0) {
            svg.select("#nodesHighlighted")
                .select("#nh" + e.s.id)
                .transition()
                .attr("r",nodeSize)
                .remove();
            e.s.highlighted = false;
        }
        e.t.highlight_degree--;
        if (e.t.highlight_degree === 0) {
            svg.select("#nodesHighlighted")
                .select("#nh" + e.t.id)
                .transition()
                .attr("r",nodeSize)
                .remove();
            e.t.highlighted = false;
        }
    }
}

function unhighlightNode(svg, v) {
    svg.select("#nodesHighlighted")
        .select("#nh" + v.id)
        .transition()
        .attr("r", nodeSize)
        .remove();
    v.highlighted = false;
}


function toggle(graph, svg, vertices, edges, e, adjList, adjMatrix, selected_path, path_start, path_target) {
    var i, s, t, sedges;
    if (e.highlighted && !e.selected) { // e is highlighted, but not selected
        // s is selected, t unselected
        if (e.s.selected && e.t.selected) return;
        if (!e.s.selected && !e.t.selected) return;
        s = e.s.selected ? e.s : e.t;
        t = e.s.selected ? e.t : e.s;

        // remove highlighted from t (handled below)
        /*t.highlighted = false;
         svg.select("#nodesHighlighted")
         .select("#nh" + t.id)
         .remove();*/

        // remove highlighted edges for s
        sedges = adjList[s.id];
        for (i = 0; i < sedges.length; i++) {
            unhighlight(svg, sedges[i])
        }

        // select t
        t.component = s.component;
        selectNode(svg, t, selected_path);

        // select e
        selectLink(svg, e);

        // check if t adjacent to endpoint of other selected path
        var u = selected_path[1 - t.component].slice(-1)[0];
        if (adjMatrix[t.id][u.id]) {
            // the path is complete
            graph.path_complete = true;
            // select also the edge between t and u
            selectLink(svg, adjMatrix[t.id][u.id]);
            // remove all highlighted nodes and links
            svg.select("#nodesHighlighted")
                .selectAll("circle")
                .remove();
            svg.select("#linksHighlighted")
                .selectAll("line")
                .remove();
            for (i = 0; i < vertices.length; i++) {
                vertices[i].highlighted = false;
                vertices[i].highlight_degree = 0;
            }
            for (i = 0; i < edges.length; i++) {
                edges[i].highlighted = false;
            }
            // fill selected_path[0], empty selected_path[1]
            while (selected_path[1].length > 0) {
                var w = selected_path[1].pop();
                selected_path[0].push(w);
                w.component = 0;
            }


        }

        // highlight new nodes and links starting from t
        // (unless path is complete)
        if (!graph.path_complete) {
            findNewHighlights(svg, t, adjList)
        }

    } else if (e.selected) { // e is selected --> remove it
        // assume that path is not complete
        if (!graph.path_complete) {
            // only allow removal of edges at the end
            // don't allow to remove an edge incident to path_start or path_target
            // unless it's the only edge in the connected component or the
            // path is complete
            if ((e.s.path_degree > 1
                || e.s.id === path_start.id
                || e.s.id === path_target.id)
                && (e.t.path_degree > 1
                || e.t.id === path_start.id
                || e.t.id === path_target.id))
                return false;
            // v is the Vertex with degree 1 other than the start or the target.
            var v = (e.s.path_degree > 1
            || e.s.id === path_start.id
            || e.s.id === path_target.id)
                ? e.t : e.s;
            // u is the other one
            u = (e.s.id === v.id) ? e.t : e.s;

            // unhighlight edges incident to v
            var vedges = adjList[v.id];
            for (i = 0; i < vedges.length; i++)
                unhighlight(svg, vedges[i])

            // unselect v and e
            v.selected = false;
            e.selected = false;
            svg.select("#nodesSelected")
                .select("#ns" + v.id)
                .remove();
            svg.select("#linksSelected")
                .select("#ls" + e.id)
                .remove();
            v.path_degree--;
            u.path_degree--;

            // remove v from its selected path
            selected_path[v.component].pop();
            v.component = -1;

            // check if there is an edge between u and the endpoint t
            // of the other selected path
            t = selected_path[1 - u.component].slice(-1)[0];
            if (adjMatrix[u.id][t.id]) {
                // the path is complete
                graph.path_complete = true;
                // select also the edge between t and u
                selectLink(svg, adjMatrix[t.id][u.id]);
                // remove all highlighted nodes and links
                svg.select("#nodesHighlighted")
                    .selectAll("circle")
                    .remove();
                svg.select("#linksHighlighted")
                    .selectAll("line")
                    .remove();
                for (i = 0; i < vertices.length; i++) {
                    vertices[i].highlighted = false;
                    vertices[i].highlight_degree = 0;
                }
                for (i = 0; i < edges.length; i++) {
                    edges[i].highlighted = false;
                }
                // fill selected_path[0], empty selected_path[1]
                while (selected_path[1].length > 0) {
                    w = selected_path[1].pop();
                    w.component = 0;
                    selected_path[0].push(w)
                }
            } else {
                // highlight new nodes and links starting from u
                findNewHighlights(svg, u, adjList)
            }
        }
    } else {
        return false;
    }
    /*var str = "paths: [";
     for (i = 0; i < selected_path[0].length; i++) {
     str += selected_path[0][i].id + ", "
     }
     str = str.slice(0, str.length-2);
     str += "]\n patht: [";
     for (i = 0; i < selected_path[1].length; i++) {
     str += selected_path[1][i].id + ", "
     }
     str = str.slice(0, str.length-2) + "]";
     addDebug(str);*/
    return true;
}

function toggleNode(graph, svg, vertices, edges, v, adjList, adjMatrix, selected_path, path_start, path_target) {
    var vedges = adjList[v.id];
    if (graph.path_complete
        && v.selected
        && v.id !== path_start.id
        && v.id !== path_target.id) {
        // path is complete, unselecting v disconnects the path
        graph.path_complete = false;
        // remove one of the selected edges from v
        var e1;
        var e2;
        for (var i = 0; i < vedges.length; i++) {
            var vedge = vedges[i];
            if (vedge.selected) {
                if (!e1) {
                    e1 = vedge;
                } else {
                    e2 = vedge;
                    break;
                }
            }
        }
        // unselect e1
        svg.select("#linksSelected")
            .select("#ls" + e1.id)
            .remove();
        e1.selected = false;
        e1.s.path_degree--;
        e1.t.path_degree--;

        // split the selected path
        var w = selected_path[0].pop();
        while (w.id !== e1.s.id && w.id !== e1.t.id) {
            w.component = 1;
            selected_path[1].push(w);
            w = selected_path[0].pop()
        }
        w.component = 1;
        selected_path[1].push(w);

        // then call graph.path_complete for the other edge from v
        toggle(graph, svg, vertices, edges, e2, adjList, adjMatrix, selected_path, path_start, path_target);

        // find new highlights from the endpointu  of e1 different from v
        u = (e1.s.id === v.id) ? e1.t : e1.s;
        findNewHighlights(svg, u, adjList);
        return true;
    }

    // try to reuse toggle
    if (v.highlighted) {
        for (i = 0; i < vedges.length; i++) {
            vedge = vedges[i];
            if (vedge.highlighted) {
                return toggle(graph, svg, vertices, edges, vedge, adjList, adjMatrix, selected_path, path_start, path_target)
            }
        }
    } else if (v.selected && v.path_degree === 1) {
        for (i = 0; i < vedges.length; i++) {
            vedge = vedges[i];
            if (vedge.selected) {
                return toggle(graph, svg, vertices, edges, vedge, adjList, adjMatrix, selected_path, path_start, path_target);
            }
        }
    } else {
        return false;
    }
}

function sqr(x) {
    return x * x
}

function dist2(v, w) {
    return sqr(v.x - w.x) + sqr(v.y - w.y)
}

function distToSegmentSquared(p, v, w) {
    var l2 = dist2(v, w);
    if (l2 === 0) return dist2(p, v);
    var t = ((p.x - v.x) * (w.x - v.x) + (p.y - v.y) * (w.y - v.y)) / l2;
    t = Math.max(0, Math.min(1, t));
    return dist2(p, {
        x: v.x + t * (w.x - v.x),
        y: v.y + t * (w.y - v.y)
    });
}

function distToSegment(p, v, w) {
    return Math.sqrt(distToSegmentSquared(p, v, w));
}

function detect(svg, vertices, edges, x, y, adjList, adjMatrix, selected_path, path_start, path_target) {
    var p = {x: x, y: y};
    console.log(p);
    var max_distance = nodeSize * 5;
    // find closest node
    var closest_node = null;
    var closest_distance = 600;
    for (var i = 0; i < vertices.length; i++) {
        var v = vertices[i];
        var dist = Math.sqrt(dist2(p, v));
        if ((v.highlighted || v.selected) && dist < closest_distance) {
            closest_node = v;
            closest_distance = dist
        }
    }
    // only select node if dist is at most selectSize
    if (closest_node && closest_distance <= selectSize) {
        toggleNode(graph, svg, vertices, edges, closest_node, adjList, adjMatrix, selected_path, path_start, path_target);
        return;
    }

    // find closest edge
    var closest_edge = null;
    var closest_edge_distance = 600;
    for (i = 0; i < edges.length; i++) {
        var e = edges[i];
        dist = distToSegment(p, e.s, e.t);
        if ((e.highlighted || e.selected) && dist < closest_edge_distance) {
            closest_edge = e;
            closest_edge_distance = dist
        }
    }
    // only select edge if dist is at most nodeSize
    if (closest_edge && closest_edge_distance <= nodeSize) {
        toggle(graph, svg, vertices, edges, closest_edge, adjList, adjMatrix, selected_path, path_start, path_target);
        return;
    }

    // is there a node or edge within max_distance?
    closest_distance -= selectSize;
    closest_edge_distance -= nodeSize;
    if (closest_distance > closest_edge_distance && closest_edge_distance <= max_distance) {
        // use the edge
        if (closest_edge.selected)
            toggle(graph, svg, vertices, edges, closest_edge, adjList, adjMatrix, selected_path, path_start, path_target);
    } else if (closest_distance <= closest_edge_distance && closest_distance <= max_distance) {
        // use the node
        toggle(graph, svg, vertices, edges, closest_edge, adjList, adjMatrix, selected_path, path_start, path_target);
    }
}